"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ObjectWatcher_1 = require("./ObjectWatcher");
var ObjectMutator_1 = require("./ObjectMutator");
var ObjectAccessor_1 = require("./ObjectAccessor");
var TypeRegistry_1 = require("../types/TypeRegistry");
/**
 * Container for change function that gets called when values in a DeltaWatch are modified
 */
var WatcherOptions = /** @class */ (function () {
    function WatcherOptions(change) {
        this._change = change;
    }
    Object.defineProperty(WatcherOptions.prototype, "change", {
        get: function () {
            return this._change;
        },
        enumerable: true,
        configurable: true
    });
    return WatcherOptions;
}());
exports.WatcherOptions = WatcherOptions;
var DeltaWatch = /** @class */ (function () {
    function DeltaWatch(data) {
        if (data === null || typeof data === 'undefined') {
            throw new TypeError("data must be an object or array");
        }
        this._rootTypeRegistry = TypeRegistry_1.TypeRegistry.defaultTypeRegistry;
        this._dataValue = data;
        this._watcher = ObjectWatcher_1.MakeObjectWatcher(this);
        this._mutator = ObjectMutator_1.makeObjectMutator(this._watcher);
    }
    DeltaWatch.watch = function (watchable, cb) {
        watchable._addWatcher(new WatcherOptions(cb));
    };
    DeltaWatch.unwatch = function (watchable, cb) {
        watchable._removeWatcher(new WatcherOptions(cb));
    };
    Object.defineProperty(DeltaWatch.prototype, "Watcher", {
        get: function () {
            return this._watcher;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeltaWatch.prototype, "Accessor", {
        get: function () {
            return ObjectAccessor_1.makeObjectAccessor(this._dataValue, this._typeRegistry);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeltaWatch.prototype, "Mutator", {
        get: function () {
            return this._mutator;
        },
        set: function (data) {
            this._dataValue = data;
            // No need to notify parent since it is the top
            this._watcher._notifySubscribers(false, true);
        },
        enumerable: true,
        configurable: true
    });
    DeltaWatch.prototype._addWatcher = function (cb) {
        this._watcher._addWatcher(cb);
    };
    DeltaWatch.prototype._removeWatcher = function (cb) {
        this._watcher._removeWatcher(cb);
    };
    Object.defineProperty(DeltaWatch.prototype, "_typeRegistry", {
        get: function () {
            return this._rootTypeRegistry;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DeltaWatch.prototype, "_data", {
        get: function () {
            return this._dataValue;
        },
        enumerable: true,
        configurable: true
    });
    return DeltaWatch;
}());
exports.DeltaWatch = DeltaWatch;
//# sourceMappingURL=DeltaWatch.js.map