"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ObjectWatcherHandler = {
    get: function (obj, prop) {
        // Intercept any get to an undefined property as creating a new watchable property on the object
        if (!(prop in obj)) {
            // Trying to access a field on an undefined data
            // Make data an object and create a dynamic property on it
            if (typeof obj._data === 'undefined'
                && obj._parent instanceof ObjectWatcher
                && typeof ObjectWatcher.getMutator(obj._parent, prop) === 'undefined') {
                // obj._parent._data[obj._parentKey] = {};
                obj._parent._makeMutator(obj._parentKey);
            }
            obj._makeProperty(prop);
            return obj._childProperties[prop];
        }
        return obj[prop];
    }
};
function MakeObjectWatcher(parent, parentKey, skipChildren) {
    if (skipChildren === void 0) { skipChildren = false; }
    return new Proxy(new ObjectWatcher(parent, parentKey, skipChildren), ObjectWatcherHandler);
}
exports.MakeObjectWatcher = MakeObjectWatcher;
var ObjectWatcher = /** @class */ (function () {
    function ObjectWatcher(parent, parentKey, skipChildren) {
        if (skipChildren === void 0) { skipChildren = false; }
        this._watcherOptions = [];
        this._parent = parent;
        this._parentKey = parentKey;
        this._lastValue = this._data;
        this._childProperties = {};
        this._mutators = {};
        // Don't make any properties
        // Proxy will handle their dynamic creation if it happens
        if (this._data !== null && typeof this._data !== 'undefined' && !skipChildren) {
            for (var _i = 0, _a = Object.keys(this._data); _i < _a.length; _i++) {
                var field = _a[_i];
                this._makeProperty(field);
            }
        }
    }
    ObjectWatcher.getMutator = function (watcher, field) {
        return watcher._mutators[field] || null;
    };
    ObjectWatcher.getFieldWatcher = function (watcher, field) {
        var w = watcher._childProperties[field];
        if (typeof w === 'undefined' || w === null) {
            watcher._makeProperty(field);
            w = watcher._childProperties[field];
        }
        return w;
    };
    ObjectWatcher.prototype._addWatcher = function (options) {
        this._watcherOptions.push(options);
    };
    ObjectWatcher.prototype._removeWatcher = function (options) {
        var index = this._watcherOptions.findIndex(function (opt) { return opt.change === options.change; });
        if (index > -1) {
            this._watcherOptions.splice(index, 1);
        }
    };
    ObjectWatcher.prototype._makeMutator = function (field) {
        var watcher = this._properties[field];
        var makeMutator = this._typeRegistry.getMakeMutatorForValue(this._data[field]);
        if (makeMutator) {
            this._mutators[field] = makeMutator(watcher);
        }
    };
    ObjectWatcher.prototype._makeProperty = function (field) {
        var self = this;
        var fieldData = this._fieldData(field);
        var dataType = typeof fieldData;
        if (fieldData !== null && dataType === "object") {
            this._properties[field] = MakeObjectWatcher(this, field);
            this._makeMutator(field);
        }
        else {
            this._properties[field] = MakeObjectWatcher(this, field, true);
        }
        // Create watcher property
        // Getter returns WatchableProperty that can be used to subscribe to
        Object.defineProperty(this, field, {
            get: function () {
                return self._properties[field];
            }
        });
    };
    ObjectWatcher.prototype._notifySubscribers = function (notifyParent, notifyChildren, fromChangedChild) {
        if (notifyParent === void 0) { notifyParent = false; }
        if (notifyChildren === void 0) { notifyChildren = false; }
        if (fromChangedChild === void 0) { fromChangedChild = false; }
        var hasChildren = Object.keys(this._childProperties).length !== 0;
        var childrenChanged = false;
        if (notifyChildren) {
            for (var _i = 0, _a = Object.keys(this._childProperties); _i < _a.length; _i++) {
                var childKey = _a[_i];
                // Continue notification downstream
                var childProps = this._childProperties[childKey];
                var changed_1 = childProps._notifySubscribers(false, true);
                if (changed_1) {
                    childrenChanged = true;
                }
            }
        }
        // Changed if value has changed
        var changed = (this._lastValue !== this._data);
        // If their are children, it doesn't matter if our value has changed
        // If any child has changed this object will notify subscribers
        // However, if the new value of this object shouldn't have children
        // (e.g. its value was an object but its new value is a primitive)
        var shouldHaveChildren = typeof (this._data) === 'object' && this._data !== null;
        if (hasChildren && shouldHaveChildren) {
            changed = childrenChanged;
        }
        // If the call was from a changed child
        // force a change (this being the parent of the changed child)
        changed = changed || fromChangedChild;
        if (changed) {
            for (var _b = 0, _c = this._watcherOptions; _b < _c.length; _b++) {
                var subscriber = _c[_b];
                if (typeof subscriber.change === 'function') {
                    subscriber.change(this._data);
                }
            }
            this._lastValue = this._data;
        }
        if (notifyParent && changed) {
            if (this._parent instanceof ObjectWatcher) {
                // Continue the notification upstream
                this._parent._notifySubscribers(true, false, true);
            }
        }
        return changed;
    };
    ObjectWatcher.prototype._fieldData = function (field) {
        return this._data ? this._data[field] : undefined;
    };
    Object.defineProperty(ObjectWatcher.prototype, "_data", {
        get: function () {
            if (this._parent._data === null || typeof this._parent._data === 'undefined') {
                return;
            }
            return this._parentKey === null || typeof this._parentKey === 'undefined' ?
                this._parent._data : this._parent._data[this._parentKey];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObjectWatcher.prototype, "_subscribers", {
        get: function () {
            return this._watcherOptions;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObjectWatcher.prototype, "_properties", {
        get: function () {
            return this._childProperties;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ObjectWatcher.prototype, "_typeRegistry", {
        get: function () {
            return this._parent._typeRegistry;
        },
        enumerable: true,
        configurable: true
    });
    return ObjectWatcher;
}());
exports.ObjectWatcher = ObjectWatcher;
//# sourceMappingURL=ObjectWatcher.js.map