"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ObjectMutator_1 = require("../core/ObjectMutator");
var ObjectAccessor_1 = require("../core/ObjectAccessor");
var ArrayType_1 = require("./ArrayType");
var DateType_1 = require("./DateType");
var TypeRegistry = /** @class */ (function () {
    function TypeRegistry(types) {
        this._types = types;
    }
    Object.defineProperty(TypeRegistry, "defaultTypeRegistry", {
        get: function () {
            return new TypeRegistry([
                DateType_1.DateTypeInfo,
                ArrayType_1.ArrayTypeInfo
            ]);
        },
        enumerable: true,
        configurable: true
    });
    TypeRegistry.prototype.getMakeAccessorForValue = function (value) {
        var accessor = null;
        for (var _i = 0, _a = this._types; _i < _a.length; _i++) {
            var info = _a[_i];
            if (info.handlesValue(value)) {
                accessor = info.makeAccessor;
                break;
            }
        }
        if (accessor === null
            && typeof value === "object"
            && value !== null) { // Don't handle null by default, if a custom type want's to handle it accessor won't be null for this check
            return ObjectAccessor_1.makeObjectAccessor;
        }
        return accessor;
    };
    TypeRegistry.prototype.getMakeMutatorForValue = function (value) {
        var mutator = null;
        for (var _i = 0, _a = this._types; _i < _a.length; _i++) {
            var info = _a[_i];
            if (info.handlesValue(value)) {
                mutator = info.makeMutator;
            }
        }
        if (mutator === null
            && (typeof value === 'object' || typeof value === 'undefined')) {
            mutator = ObjectMutator_1.makeObjectMutator;
        }
        return mutator;
    };
    TypeRegistry.prototype.getTypeForValue = function (value) {
        var type = "";
        for (var _i = 0, _a = this._types; _i < _a.length; _i++) {
            var info = _a[_i];
            if (info.handlesValue(value)) {
                type = info.type;
            }
        }
        if (type === "" && typeof value === "object") {
            type = "Object";
        }
        return type;
    };
    return TypeRegistry;
}());
exports.TypeRegistry = TypeRegistry;
//# sourceMappingURL=TypeRegistry.js.map