"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ObjectWatcher_1 = require("../core/ObjectWatcher");
function makeAccessorHandler(mutatorMethods) {
    return {
        get: function (obj, prop) {
            if (prop in obj) {
                var field = obj[prop];
                if (typeof field === 'function') {
                    if (mutatorMethods.indexOf(prop) !== -1) {
                        throw Error("Cannot access a mutator method on an Accessor object.");
                    }
                    field = field.bind(obj);
                }
                return field;
            }
        },
        set: function () {
            throw Error("Cannot set a value on the Accessor object.");
        }
    };
}
exports.makeAccessorHandler = makeAccessorHandler;
function makeMutationHandler(internals, mutatorMethods, canSetProp) {
    if (canSetProp === void 0) { canSetProp = false; }
    return {
        get: function (_, prop) {
            if (prop === "__DeltaWatchInternals") {
                return internals;
            }
            var fieldMutator = ObjectWatcher_1.ObjectWatcher.getMutator(internals.watcher, prop);
            var fieldWatcher = ObjectWatcher_1.ObjectWatcher.getFieldWatcher(internals.watcher, prop);
            if (prop in internals.watcher._data) {
                var field_1 = internals.watcher._data[prop];
                if (typeof field_1 === 'function') {
                    field_1 = field_1.bind(internals.watcher._data);
                    if (mutatorMethods.indexOf(prop) !== -1) {
                        return function () {
                            var args = [];
                            for (var _i = 0; _i < arguments.length; _i++) {
                                args[_i] = arguments[_i];
                            }
                            var result = field_1.apply(void 0, args);
                            internals.watcher._notifySubscribers(true, true, true);
                            return result;
                        };
                    }
                }
                else {
                    if (fieldMutator !== null && typeof fieldMutator !== 'undefined') {
                        return fieldMutator;
                    }
                    else if (fieldWatcher !== null && typeof fieldWatcher !== 'undefined') {
                        internals.watcher._makeMutator(prop);
                        return ObjectWatcher_1.ObjectWatcher.getMutator(internals.watcher, prop);
                    }
                }
                return field_1;
            }
            else if (fieldMutator !== null && typeof fieldMutator !== 'undefined') {
                return fieldMutator;
            }
            else if (fieldWatcher !== null && typeof fieldWatcher !== 'undefined') {
                internals.watcher._makeMutator(prop);
                return ObjectWatcher_1.ObjectWatcher.getMutator(internals.watcher, prop);
            }
        },
        set: function (_, prop, value) {
            if (prop === "__DeltaWatchInternals") {
                throw Error("Cannot set value of __DeltaWatchInternals");
            }
            var willSetProp;
            if (canSetProp instanceof Function) {
                willSetProp = canSetProp(prop);
            }
            else {
                willSetProp = canSetProp;
            }
            // If any prop should be accepted as settable
            // or settableProps contains the given prop
            if (willSetProp === true) {
                // Need to mutate before array check,
                // because watcher's make mutator uses current value to determine
                // to make Object or Array mutator
                // if data is undefined then we need to make it an object
                if (typeof internals.watcher._data === 'undefined') {
                    internals.watcher._parent._data[internals.watcher._parentKey] = {};
                }
                internals.watcher._data[prop] = value;
                var fieldMutator = ObjectWatcher_1.ObjectWatcher.getMutator(internals.watcher, prop);
                var type = internals.watcher._typeRegistry.getTypeForValue(value);
                if (Array.isArray(value) &&
                    (fieldMutator === null || typeof fieldMutator === 'undefined'
                        || fieldMutator.__DeltaWatchInternals.type !== type)) {
                    // if mutator hasn't been initialized
                    // or the type of the mutator doesn't match the new value
                    // make a new mutator for it
                    internals.watcher._makeMutator(prop);
                }
                var watcherProperties = internals.watcher._properties[prop];
                if (watcherProperties) {
                    watcherProperties._notifySubscribers(true, true);
                }
                return true;
            }
            throw Error(prop + " not settable on Mutator of type " + internals.type);
        }
    };
}
exports.makeMutationHandler = makeMutationHandler;
//# sourceMappingURL=utils.js.map